#version 120

#define SCANLINE_INTENSITY 0.5 //[0.0 0.25 0.5 0.75 1.0]
#define NOISE_INTENSITY 0.2 //[0.0 0.1 0.2 0.3 0.4 0.5]
#define CHROMATIC_ABERRATION_INTENSITY 0.005 //[0.0 0.002 0.005 0.008 0.01]
#define NOISE_SPEED 1.0 //[0.5 1.0 1.5 2.0]

uniform sampler2D texture;
uniform float frameTimeCounter;

varying vec2 texCoord;
varying vec4 vertexColor;

float rand(vec2 co) {
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main() {
    vec2 uv = texCoord;
    
    vec2 redOffset = vec2(CHROMATIC_ABERRATION_INTENSITY, 0.0);
    vec2 greenOffset = vec2(0.0, 0.0);
    vec2 blueOffset = vec2(-CHROMATIC_ABERRATION_INTENSITY, 0.0);
    
    vec4 colorDistortion;
    colorDistortion.r = texture2D(texture, uv + redOffset).r;
    colorDistortion.g = texture2D(texture, uv + greenOffset).g;
    colorDistortion.b = texture2D(texture, uv + blueOffset).b;
    colorDistortion.a = texture2D(texture, uv).a;
    
    float scanline = sin(uv.y * 800.0) * 0.5 + 0.5;
    colorDistortion.rgb *= mix(1.0, scanline, SCANLINE_INTENSITY);
    
    vec2 noiseUV = uv + vec2(frameTimeCounter * 0.1 * NOISE_SPEED, frameTimeCounter * 0.2 * NOISE_SPEED);
    float noise = rand(noiseUV);

    colorDistortion.rgb = mix(colorDistortion.rgb, vec3(noise), NOISE_INTENSITY);
    
    gl_FragColor = colorDistortion * vertexColor;
}

