#version 120

/* 
 * COMPOSITE: BARREL_DISTORTION_ONLY
 * - Barrel Distortion
 */

// Shader Options
/* DRAWBUFFERS:0 */

#define DISTORTION_OPTION 1 // [0 1] 1 to enable distortion, 0 to disable distortion
#define BARREL_POWER 0.1 // [0.0 0.1 0.2 0.3 0.4 0.5] Distortion strength

uniform sampler2D colortex0;
uniform mat4 gbufferProjection;
uniform float viewWidth;
uniform float viewHeight;

varying vec2 texcoord;

// Function to apply barrel distortion
vec2 distort(vec2 uv, float power) {
    // Normalize coordinates to center (0,0)
    vec2 centered = uv - 0.5;
    float dist = length(centered);
    // Apply distortion
    float distortion = 1.0 + power * dist * dist;
    return centered * distortion + 0.5;
}

void main() {
    vec2 uv = texcoord;

    #if DISTORTION_OPTION
        // Calculate Field of View (FOV) based on projection matrix
        float fov = 2.0 * atan(1.0 / gbufferProjection[0][0]);

        // Apply barrel distortion
        uv = distort(uv, BARREL_POWER * fov * 0.5);

        // Clip the UV coordinates to remain within [0,1]
        if (uv.x < 0.0 || uv.x > 1.0 || uv.y < 0.0 || uv.y > 1.0) {
            gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
            return;
        }
    #endif

    // Sample the texture with the potentially distorted UV coordinates
    vec3 color = texture2D(colortex0, uv).rgb;
    
    gl_FragColor = vec4(color, 1.0);
}
