#version 120

#include "/settings.glsl"

// Celshading (1 = enable, 0 = disable)
#define ENABLE_CELSHADING 1
#define DISABLE_CELSHADING 0
#define TOGGLE_CELSHADING DISABLE_CELSHADING // [DISABLE_CELSHADING ENABLE_CELSHADING]

varying vec4 texcoord;
uniform sampler2D gcolor;
uniform sampler2D depthtex0;
uniform float viewWidth;
uniform float viewHeight;
uniform float near;
uniform float far;

vec3 celshade(vec3 color) {
    float d = texture2D(depthtex0, texcoord.xy).r;
    float dtresh = 1.0 / (far - near) / 50.0;    
    vec4 dc = vec4(d);
    vec4 sa, sb;
    float pw = 1.0 / viewWidth;
    float ph = 1.0 / viewHeight;
    float BORDERC = 1.0;
    
    sa.x = texture2D(depthtex0, texcoord.xy + vec2(-pw, -ph) * BORDERC).r;
    sa.y = texture2D(depthtex0, texcoord.xy + vec2(pw, -ph) * BORDERC).r;
    sa.z = texture2D(depthtex0, texcoord.xy + vec2(-pw, 0.0) * BORDERC).r;
    sa.w = texture2D(depthtex0, texcoord.xy + vec2(0.0, ph) * BORDERC).r;
    
    sb.x = texture2D(depthtex0, texcoord.xy + vec2(pw, ph) * BORDERC).r;
    sb.y = texture2D(depthtex0, texcoord.xy + vec2(-pw, ph) * BORDERC).r;
    sb.z = texture2D(depthtex0, texcoord.xy + vec2(pw, 0.0) * BORDERC).r;
    sb.w = texture2D(depthtex0, texcoord.xy + vec2(0.0, -ph) * BORDERC).r;
    
    vec4 dd = abs(2.0 * dc - sa - sb) - dtresh;
    dd = step(dd, vec4(0.0));
    
    float e = clamp(dot(dd, vec4(0.25)), 0.0, 1.0);
    return color * e;
}

void main() {
    vec3 color = texture2D(gcolor, texcoord.xy).rgb;
    
    #if TOGGLE_CELSHADING == ENABLE_CELSHADING
        color = celshade(color);
    #endif

    /* DRAWBUFFERS:0 */
    gl_FragData[0] = vec4(color, 1.0);
}
