#version 460

uniform sampler2D colortex0;

in vec2 texCoord;

layout(location = 0) out vec4 outColor0;

#define LUMA_COMPRESSION 8 //[1 2 3 4 5 6 7 8]

vec3 rgb2yuv(vec3 rgb) {
    float y = 0.299 * rgb.r + 0.587 * rgb.g + 0.114 * rgb.b;
    return vec3(y, 0.493 * (rgb.b - y), 0.877 * (rgb.r - y));
}

vec3 yuv2rgb(vec3 yuv) {
    float y = yuv.x;
    float u = yuv.y;
    float v = yuv.z;
    
    return vec3(
        y + 1.0 / 0.877 * v,
        y - 0.39393 * u - 0.58081 * v,
        y + 1.0 / 0.493 * u
    );
}

void main() {
    vec4 inputColour = texture(colortex0, texCoord);

    float y = rgb2yuv(textureLod(colortex0, texCoord, LUMA_COMPRESSION).rgb).x;
    vec2 uv = rgb2yuv(inputColour.rgb).yz;
    vec3 rgb = yuv2rgb(vec3(y, uv));
    inputColour.rgb = rgb;

    outColor0 = inputColour;
}
