#version 120

#define BLOOM_STRENGTH 0.2 //[0.0 0.2 0.4 0.5 0.6 0.8 1.0]
#define GAMMA_ADJUSTMENT 0.2 //[0.0 0.1 0.2 0.3 0.4 0.5]
#define BLOOM_OFF 0
#define BLOOM_ON 1
#define BLOOM_TOGGLE BLOOM_ON //[BLOOM_OFF BLOOM_ON]

uniform sampler2D gcolor;
uniform float viewWidth;
uniform float viewHeight;

varying vec2 texcoord;

vec3 getBloomColor(vec2 texCoords) {
    vec3 bloomColor = vec3(0.0);
    float totalWeight = 0.0;
    
    for (int x = -4; x <= 4; x++) {
        for (int y = -4; y <= 4; y++) {
            vec2 offset = vec2(float(x), float(y)) / vec2(viewWidth, viewHeight);
            float weight = 1.0 / (1.0 + float(x*x + y*y));
            bloomColor += texture2D(gcolor, texCoords + offset).rgb * weight;
            totalWeight += weight;
        }
    }
    
    return bloomColor / totalWeight;
}

void main() {
    vec3 color = texture2D(gcolor, texcoord).rgb;
    
    #if BLOOM_TOGGLE == BLOOM_ON
        vec3 bloomColor = getBloomColor(texcoord);
        color += bloomColor * BLOOM_STRENGTH;
    #endif
    
    // Apply gamma adjustment
    color = pow(color, vec3(1.0 / (1.0 + GAMMA_ADJUSTMENT)));
    
    gl_FragColor = vec4(color, 1.0);
}

