#version 120
#include "/settings/lib.h"

uniform sampler2D lightmap;
uniform sampler2D texture;
uniform int isEyeInWater;
uniform float far;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform vec3 skyColor;
uniform vec3 fogColor;
uniform vec4 entityColor;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 cPos;
varying vec3 wPos;
varying vec3 sunPos;
varying vec3 glnormal;
varying vec4 glcolor;
varying float isGlint; // Add this varying for glint

void main() {
    vec4 diffuse = texture2D(texture, texcoord) * glcolor;
    diffuse.rgb = mix(diffuse.rgb, entityColor.rgb, entityColor.a);

    // Basic lighting
    float indoor = smoothstep(.95, 1., lmcoord.y);
    float cave = smoothstep(.7, 1., lmcoord.y);
    float day = saturate(skyColor.r * 2.);
    float dusk = saturate((fogColor.r - .1) - fogColor.b);
    float mainShadow = smoothstep(.95, .96, lmcoord.y);
    float sideShadow = smoothstep(.64, .68, glcolor.g);

    #ifdef SHADOWS
    float shadowDepth = mix(mix(1., .73, day), 1., lmcoord.x * 1.2);
    diffuse.rgb *= mix(shadowDepth, 1., mainShadow);
    diffuse.rgb *= mix(1., mix(shadowDepth, 1., saturate(dot(sunPos, glnormal))), mainShadow);
    #endif

    // Enchantment glint effect
    #ifdef HAND_GLINT
    if (isGlint > 0.5) {
        // Animated glint pattern
        float glintPattern = sin(frameTimeCounter * 4.0 + 
                               texcoord.x * 10.0 + 
                               texcoord.y * 10.0) * 0.5 + 0.5;
        
        // Scrolling effect
        vec2 glintCoord = texcoord * 2.0 + frameTimeCounter * 0.2;
        float scrollPattern = noise(glintCoord) * 0.5 + 0.5;
        
        // Combine patterns
        float glintFactor = glintPattern * scrollPattern;
        
        // Add glint color
        vec3 glintColor = vec3(0.9, 0.6, 1.0); // Purple-ish enchant color
        diffuse.rgb += glintColor * glintFactor * 0.5;
    }
    #endif

    gl_FragData[0] = diffuse;
}
