#version 120
#include "/settings/lib.h"

#define FOGSTART 0.2 //[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define FOGEND 0.5 //[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]

uniform sampler2D lightmap;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowtex0;
uniform sampler2D shadowtex1;
uniform sampler2D texture;
uniform int isEyeInWater;
uniform float far;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform vec3 skyColor;
uniform vec3 fogColor;

varying float isPlant;
varying float isOre;
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 cPos;
varying vec3 wPos;
varying vec3 sunPos;
varying vec3 normal;
varying vec4 position;
varying vec4 glcolor;
varying vec4 shadowPos;

const bool shadowcolor0Nearest = true;
const bool shadowtex0Nearest = true;
const bool shadowtex1Nearest = true;

void main() {
    vec4 diffuse = texture2D(texture, texcoord) * glcolor;
    vec2 lm = lmcoord;

    #ifdef UNDERWATER
    float lmy = 1.;
    if (isEyeInWater != 1) lmy = lmcoord.y;
    diffuse *= texture2D(lightmap, vec2(lmcoord.x, lmy));
    #else
    diffuse *= texture2D(lightmap, lmcoord);
    #endif

    // Data Base
    float indoor = smoothstep(0.95, 1., lmcoord.y);
    float cave = smoothstep(0.7, 1., lmcoord.y);
    float day = saturate(skyColor.r * 2.);
    float dusk = saturate((fogColor.r - 0.1) - fogColor.b);
    float mainShadow = smoothstep(0.95, 0.96, lmcoord.y);
    float sideShadow = smoothstep(0.64, 0.68, glcolor.g);


    // Additional shadow effects
    #ifdef SHADOWS
    float shadowDepth = mix(mix(1., 0.84, day), 1., lmcoord.x * 4.2);
    diffuse.rgb *= mix(shadowDepth, 1., mainShadow);
    if (glcolor.r == glcolor.g && glcolor.g == glcolor.b) 
        diffuse.rgb *= mix(mix(1., mix(shadowDepth, 1., saturate(dot(sunPos, normal))), mainShadow), 1., sideShadow);
    if (isPlant > 0.5) 
        diffuse.rgb *= mix(1., mix(shadowDepth, 1., saturate(dot(sunPos, normal))), mainShadow);
    #endif

    // Fog calculation
    float dist = length(position.xyz);
    float fogFactor = clamp((dist - far * FOGSTART) / (far * (FOGEND - FOGSTART)), 0.0, 1.0);
    vec3 fogColorFinal = isEyeInWater == 1 ? vec3(0.0, 0.1, 0.2) : fogColor;

    // Apply fog
    diffuse.rgb = mix(diffuse.rgb, fogColorFinal, fogFactor);

    gl_FragData[0] = diffuse;
}
