#version 120
#include "/settings/lib.h"

#define FOGSTART 0.2 //[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define FOGEND 0.5 //[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define COLORED_SHADOWS 1 //[0 1 2]

attribute vec4 mc_Entity;

uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;
uniform vec3 shadowLightPosition;
uniform vec3 sunPosition;
uniform vec3 cameraPosition;
uniform float frameTimeCounter;

varying float isPlant;
varying float isOre;
varying vec2 lmcoord;
varying vec2 texcoord;
varying vec3 cPos;
varying vec3 wPos;
varying vec3 sunPos;
varying vec3 normal;
varying vec4 position;
varying vec4 glcolor;
varying vec4 shadowPos;

void main() {
    isPlant = 0.;
    isOre = 0.;

    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    glcolor = gl_Color;
    normal = gl_NormalMatrix * gl_Normal;

    float e = mc_Entity.x;
    
    // Plant detection
    if (e == 141. || e == 142. || e == 59. || e == 106. || e == 127. || 
        e == 175. || e == 83. || e == 32. || e == 37. || e == 38. || 
        e == 31. || e == 115. || e == 6. || e == 39. || e == 40. || 
        e == 104. || e == 105.) isPlant = 1.;
    
    // Ore detection with additional ores
    if (e == 56. || e == 73. || e == 129. || e == 14. || e == 15. || 
        e == 21. || e == 79. || mc_Entity.x == 11116 || mc_Entity.x == 11117 || 
        mc_Entity.x == 11118 || mc_Entity.x == 11119 || mc_Entity.x == 11120 || 
        mc_Entity.x == 11121 || mc_Entity.x == 11122 || mc_Entity.x == 11123 || 
        mc_Entity.x == 11124 || mc_Entity.x == 11125 || mc_Entity.x == 11128 || 
        mc_Entity.x == 11129 || mc_Entity.x == 11132 || mc_Entity.x == 11133 || 
        mc_Entity.x == 11136 || mc_Entity.x == 11137) isOre = 1.;

    // Position calculations
    sunPos = (vec4(sunPosition, 1.) * gl_ModelViewMatrix).xyz;
    vec4 pos = gbufferModelViewInverse * gl_ModelViewMatrix * gl_Vertex;
    cPos = pos.xyz + cameraPosition;
    
    // Simplified shadow position calculation
    float lightDot = dot(normalize(shadowLightPosition), normalize(gl_NormalMatrix * gl_Normal));
    
    if (lightDot > 0.0) {
        vec4 playerPos = gbufferModelViewInverse * gl_ModelViewMatrix * gl_Vertex;
        shadowPos = shadowProjection * (shadowModelView * playerPos);
        shadowPos.xyz = shadowPos.xyz * 0.5 + 0.5;
    }
    else {
        shadowPos = vec4(0.0);
    }
    shadowPos.w = lightDot;

    // Wave animation
    #ifdef WAVE
    float wind = cos(frameTimeCounter * 5.0 + cPos.x * 0.25 + cPos.y * 1.2 + cPos.z * 0.08);

    if (isPlant > 0.5) 
        pos.xz += cos(frameTimeCounter * 6.0 + dot(cPos, vec3(4.5, 1.6, 2.1))) * 0.05 * wind * lmcoord.y;
    
    if (e == 18. || e == 161.) 
        pos.xz += cos(frameTimeCounter * 6.0 + dot(cPos, vec3(2.5, 0.0, 1.2))) * 0.05 * wind * lmcoord.y;
    #endif

    wPos = pos.xyz;
    position = gl_ModelViewMatrix * gl_Vertex;

    gl_Position = gl_ProjectionMatrix * gbufferModelView * pos;
}
